;***************************************************************************
;* A P P L I C A T I O N   N O T E   F O R   T H E   A V R   F A M I L Y
;* 
;* Number		:AVR000
;* File Name		:"1200def.inc"
;* Title		:Register/Bit Definitions for the AT90S1200
;* Date	 		:97.12.11
;* Version		:1.22
;* Support telephone	:+47 72 88 43 88 (ATMEL Norway)
;* Support fax		:+47 72 88 43 99 (ATMEL Norway)
;* Support E-Mail	:avr@atmel.com
;* Target MCU		:AT90S1200
;*
;* DESCRIPTION
;* When including this file in the assembly program file, all I/O register
;* names and I/O register bit names appearing in the data book can be used.
;* 
;* The Register names are represented by their hexadecimal addresses.
;* 
;* The Register Bit names are represented by their bit number (0-7).
;* 
;* Please observe the difference in using the bit names with instructions
;* such as "sbr"/"cbr" (set/clear bit in register) and "sbrs"/"sbrc" 
;* (skip if bit in register set/cleared). The following example illustrates
;* this:
;* 
;* in	r16,PORTB		;read PORTB latch
;* sbr	r16,(1<<PB6)+(1<<PB5)	;set PB6 and PB5 (use masks, not bit#)
;* out  PORTB,r16		;output to PORTB
;*
;* in	r16,TIFR		;read the Timer Interrupt Flag Register
;* sbrc	r16,TOV0		;test the overflow flag (use bit#)
;* rjmp	TOV0_is_set		;jump if set
;* ...				;otherwise do something else
;***************************************************************************

;***** Specify Device
.device AT90S1200

;***** I/O Register Definitions
.equ	SREG	=$3f	;Status REGister
.equ	GIMSK	=$3b	;General Interrupt MaSK register
.equ	TIMSK	=$39	;Timer/counter Interrupt MaSK register
.equ	TIFR	=$38	;Timer/counter Interrupt Flag Register
.equ	MCUCR	=$35	;MCU general Control Register
.equ	TCCR0	=$33	;Timer/Counter 0 Control Register
.equ	TCNT0	=$32	;Timer/Counter 0 (8 bits)
.equ	WDTCR	=$21	;WatchDog Timer Control Register
.equ	EEAR	=$1e	;EEprom Address Register
.equ	EEDR	=$1d	;EEprom Data Register
.equ	EECR	=$1c	;EEprom Control Register
.equ	PORTB	=$18	;data register, PORT B
.equ	DDRB	=$17	;Data Direction Register, port B
.equ	PINB	=$16	;input PINs, port B
.equ	PORTD	=$12	;data register, PORT D
.equ	DDRD	=$11	;Data Direction Register, port D
.equ	PIND	=$10	;input PINs, port D
.equ	ACSR	=$08	;Analog comparator Control and Status Register

;***** Bit Definitions

.equ	INT0	=6	;external INTerrupt request 0 enable (in GIMSK)

.equ	TOIE0	=1	;Timer/counter 0 Overflow Interrupt Enable (in TIMSK)

.equ	TOV0	=1	;Timer/counter 0 Overflow flag (in TIFR)

.equ	SE	=5	;Sleep Enable (in MCUCR)
.equ	SM	=4	;Sleep Mode (in MCUCR)
.equ	ISC01	=1	;Interrupt Sense Control 0, bit 1 (in MCUCR)
.equ	ISC00	=0	;Interrupt Sense Control 0, bit 0 (in MCUCR)

.equ	CS02	=2	;Clock Select 0, bit 2 (in TCCR0)
.equ	CS01	=1	;Clock Select 0, bit 1 (in TCCR0)
.equ	CS00	=0	;Clock Select 0, bit 0 (in TCCR0)

.equ	WDE	=3	;WatchDog Enable (in WDTCR)
.equ	WDP2	=2	;WatchDog timer Prescaler, bit 2 (in WDTCR)
.equ	WDP1	=1	;WatchDog timer Prescaler, bit 1 (in WDTCR)
.equ	WDP0	=0	;WatchDog timer Prescaler, bit 0 (in WDTCR)

.equ	EEWE	=1	;EEprom Write Enable (in EECR)
.equ	EERE	=0	;EEprom Read Enable (in EECR)

.equ	PB7	=7	;Port B data register, bit 7 (in PORTB)
.equ	PB6	=6	;Port B data register, bit 6 (in PORTB)
.equ	PB5	=5	;Port B data register, bit 5 (in PORTB)
.equ	PB4	=4	;Port B data register, bit 4 (in PORTB)
.equ	PB3	=3	;Port B data register, bit 3 (in PORTB)
.equ	PB2	=2	;Port B data register, bit 2 (in PORTB)
.equ	PB1	=1	;Port B data register, bit 1 (in PORTB)
.equ	PB0	=0	;Port B data register, bit 0 (in PORTB)

.equ	DDB7	=7	;Data Direction register port B, bit 7 (in DDRB)
.equ	DDB6	=6	;Data Direction register port B, bit 6 (in DDRB)
.equ	DDB5	=5	;Data Direction register port B, bit 5 (in DDRB)
.equ	DDB4	=4	;Data Direction register port B, bit 4 (in DDRB)
.equ	DDB3	=3	;Data Direction register port B, bit 3 (in DDRB)
.equ	DDB2	=2	;Data Direction register port B, bit 2 (in DDRB)
.equ	DDB1	=1	;Data Direction register port B, bit 1 (in DDRB)
.equ	DDB0	=0	;Data Direction register port B, bit 0 (in DDRB)

.equ	PINB7	=7	;input PIN port B, bit 7 (in PINB)
.equ	PINB6	=6	;input PIN port B, bit 6 (in PINB)
.equ	PINB5	=5	;input PIN port B, bit 5 (in PINB)
.equ	PINB4	=4	;input PIN port B, bit 4 (in PINB)
.equ	PINB3	=3	;input PIN port B, bit 3 (in PINB)
.equ	PINB2	=2	;input PIN port B, bit 2 (in PINB)
.equ	PINB1	=1	;input PIN port B, bit 1 (in PINB)
.equ	PINB0	=0	;input PIN port B, bit 0 (in PINB)

.equ	PD6	=6	;Port D data register, bit 6 (in PORTD)
.equ	PD5	=5	;Port D data register, bit 5 (in PORTD)
.equ	PD4	=4	;Port D data register, bit 4 (in PORTD)
.equ	PD3	=3	;Port D data register, bit 3 (in PORTD)
.equ	PD2	=2	;Port D data register, bit 2 (in PORTD)
.equ	PD1	=1	;Port D data register, bit 1 (in PORTD)
.equ	PD0	=0	;Port D data register, bit 0 (in PORTD)

.equ	DDD6	=6	;Data Direction register port D, bit 6 (in DDRD)
.equ	DDD5	=5	;Data Direction register port D, bit 5 (in DDRD)
.equ	DDD4	=4	;Data Direction register port D, bit 4 (in DDRD)
.equ	DDD3	=3	;Data Direction register port D, bit 3 (in DDRD)
.equ	DDD2	=2	;Data Direction register port D, bit 2 (in DDRD)
.equ	DDD1	=1	;Data Direction register port D, bit 1 (in DDRD)
.equ	DDD0	=0	;Data Direction register port D, bit 0 (in DDRD)

.equ	PIND6	=6	;input PIN port D, bit 6 (in PIND)
.equ	PIND5	=5	;input PIN port D, bit 5 (in PIND)
.equ	PIND4	=4	;input PIN port D, bit 4 (in PIND)
.equ	PIND3	=3	;input PIN port D, bit 3 (in PIND)
.equ	PIND2	=2	;input PIN port D, bit 2 (in PIND)
.equ	PIND1	=1	;input PIN port D, bit 1 (in PIND)
.equ	PIND0	=0	;input PIN port D, bit 0 (in PIND)

.equ	ACD	=7	;Analog Comparator Disable (in ACSR)
.equ	ACO	=5	;Analog Comparator Output (in ACSR)
.equ	ACI	=4	;Analog Comparator Interrupt flag (in ACSR)
.equ	ACIE	=3	;Analog Comparator Interrupt Enable (in ACSR)
.equ	ACIS1	=1	;Analog Comparator Interrupt mode Select, bit 1 (in ACSR)
.equ	ACIS0	=0	;Analog Comparator Interrupt mode Select, bit 0 (in ACSR)

.equ	XRAMEND =0	;eXtended RAM END address
.equ	E2END	=3F	;E2prom END address
.equ	FLASHEND=1FF	;FLASH eeprom END address

.equ	INT0addr=$001	;External Interrupt0 Vector Address
.equ	OVF0addr=$002	;Overflow0 Interrupt Vector Address
.equ	ACIaddr =$003	;Analog Comparator Interrupt Vector Address

.def	ZL	=r30	;Z Low pointer register
