;****************** HOUSE BURGLAR ALARM / ANTIFURTO PER CASA *******************
;*
;* Title/titolo:	House burglar alarm/Antifurto per casa
;* Version/Versione:	2.0 (version for Elettronica In/
;*			     versione per Elettronica In)
;* Author/Autore:	Guido Bertolotti - guidobertolotti@yahoo.com
;* Target/Destinazione:	AT90S1200
;*
;*                      *** Main file to compile is antif.asm ***
;*                 *** Il file principale da compilare  antif.asm ***
;*
;*******************************************************************************

;*==============================================================================
;* Alarm state machine/Macchina a stati di allarme
;*==============================================================================


;*==============================================================================
;* Definition of alarm machine states/
;* Definizione degli stati della macchina di allarme
;*==============================================================================
;Warning: states must not be renumbered for the correct timer's eeprom address
;         calculation/
;Attenzione: gli stati non devono essere rinumerati per un corretto calcolo degli
;            indirizzi dei temporizzatori in EEPROM

.equ	s_inactive_output_diagn	= 0	;Inactive output diagnostic status/
					;stato Inattivo di emissione diagnostica
.equ	s_inactive		= 1	;Inactive status/stato Inattivo
.equ	s_exit			= 2	;Exit status/stato Uscita
.equ	s_guard			= 3	;Guard status/stato Guardia
.equ	s_pre_alarm		= 4	;Pre-alarm status/stato Preallarme
.equ	s_alarm			= 5	;Alarm status/stato Allarme
.equ	s_programming_exit	= 6	;Programming Exit timer status/
					;stato Programmazione temporizzazione
					;di Uscita
.equ	s_programming_guard	= 7	;Programming unused Guard timer status/
					;stato Programmazione temporizzazione
					;inutilizzata di Guardia
.equ	s_programming_pre_alarm	= 8	;Programming Pre_alarm timer status/
					;stato Programmazione temporizzazione
					;di Preallarme
.equ	s_programming_alarm	= 9	;Programming Alarm timer status/
					;stato Programmazione temporizzazione
					;di Allarme
.equ	s_programming_auto_guard=10	;Programming automatic Guard status/
					;stato Programmazione Guardia automatica


.macro m_alarm_init
;*==============================================================================
;* Alarm initialization/Inizializzazione Allarme
;*==============================================================================
	;*----------------------------------------------------------------------
	;* Port B initialization/Inizializzazione porta B
	;*----------------------------------------------------------------------
;* Unnecessary instructions if PORTB is all inputs:/
;* Istruzioni inutili se la porta B  tutta di ingressi:
;*	ldi	z_tmp,s_portb_outputs
;*	out	DDRB,z_tmp	;set port B as inputs + outputs/
;*				;assegna la porta B come ingressi + uscite
;* Unnecessary instructions if PORTB is all inputs without pull-up:/
;* Istruzioni inutili se la porta B  tutta di ingressi senza pull-up:
;*	ldi	z_tmp,s_portb_inputs
;*	out	PORTB,z_tmp	;set all port B inputs with pullup/
;*				;assegna tutti gli ingressi della porta B con
;*				;pull-up

	;*----------------------------------------------------------------------
	;* Variables initialization/Inizializzazione variabili
	;*----------------------------------------------------------------------
	rcall	l_programming	;initial status = Programming/
				;stato iniziale = Programmazione
	in	z_pinsb_val,PINB;reads initial status of port B inputs/
				;legge stato iniziale ingressi porta B
	mov	z_pinsb_new,z_pinsb_val

.endmacro

;*==============================================================================
;* Siren macros and procedures/Macro e procedure della sirena
;*==============================================================================
	;*----------------------------------------------------------------------
	;* Siren macros/Macro della sirena
	;*----------------------------------------------------------------------

.macro m_siren_on
	sbi	PORTD,b_siren	;siren on/sirena accesa
.endmacro

.macro m_siren_off
	cbi	PORTD,b_siren	;siren off/sirena spenta
.endmacro


.macro m_alarm_timer
;*==============================================================================
;* Alarm status machine timer handling/
;* Gestione temporizzazioni macchina a stati di allarme
;*==============================================================================

	;*----------------------------------------------------------------------
	;* Needs:/Ha bisogno di:
	;*----------------------------------------------------------------------
;To be called every second/Essere chiamata ogni secondo

;*==============================================================================
;* Check alarm timer: 1 s .. 18 h 12 m 15 s/
;* Controlla il temporizzatore di allarme: 1 s .. 18 h 12 m 15 s
;*==============================================================================
	mov	z_tmp,z_alarm_timer_0
				;if alarm timer active/
				;se temporizzatore di allarme attivo
	or	z_tmp,z_alarm_timer_1
	breq	l_end_alarm_tim	;then/allora
	subi	z_alarm_timer_0,1
				;  decrement alarm timer/
				;  decrementa temporizzatore di allarme
	sbci	z_alarm_timer_1,0
				;  if timer expired/
				;  se temporizzazione terminata
	brne	l_end_alarm_tim	;  then/allora

	;*----------------------------------------------------------------------
	;* Status INACTIVE/Stato INATTIVO
	;*----------------------------------------------------------------------
	cpi	z_status,s_inactive
				;    if status = inactive/se stato = inattivo
	brlo	l_output_diagn	;
	brne	l_end_check_inac;    then/allora
	rcall	l_guard		;      enter status guard/entra in stato Guardia
	rjmp	l_end_alarm_tim	;
l_output_diagn:			;    elsif status = inactive output diagn/
				;    altrimenti se stato = inattivo di emissione
				;      diagnostica
				;    then/allora
	rcall	l_output_mem_next_bit
				;      output next bit/emetti il prossimo bit
	rjmp	l_end_alarm_tim	;
l_end_check_inac:		;

	;*----------------------------------------------------------------------
	;* Status EXIT/Stato USCITA
	;*----------------------------------------------------------------------
	cpi	z_status,s_exit	;    elsif status = exit/
				;    altrimenti se stato = uscita
	brne	l_end_check_exit;    then/allora
	clr	z_tmp1		;
	ldi	z_tmp,e_sensors_in_alarm
	rcall	l_write_eeprom	;      reset sensors in alarm/
				;      azzera sensori in allarme
	ldi	z_tmp,e_power_status
	rcall	l_write_eeprom	;      reset power supply anomalies/
				;      azzera anomalie di alimentazione
	rcall	l_guard		;      enter status guard/
				;      entra in stato guardia
	rjmp	l_end_alarm_tim	;
l_end_check_exit:		;

	;*----------------------------------------------------------------------
	;* Status PRE-ALARM/Stato PREALLARME
	;*----------------------------------------------------------------------
	cpi	z_status,s_pre_alarm
				;    elsif status = pre-alarm/
				;    altrimenti se stato = preallarme
	brne	l_end_check_pre_;    then/allora
	rcall	l_alarm		;      enter status alarm/
				;      entra in stato allarme
	rjmp	l_end_alarm_tim	;
l_end_check_pre_:		;

	;*----------------------------------------------------------------------
	;* Status ALARM/Stato ALLARME
	;*----------------------------------------------------------------------
	cpi	z_status,s_alarm
				;    elsif status = alarm/
				;    altrimenti se stato = allarme
	brne	l_end_check_alar;    then/allora
	rcall	l_guard		;      enter status guard (turning siren off)/
				;      entra in stato guardia (spegnendo sirena)
	rjmp	l_end_alarm_tim	;
l_end_check_alar:		;

	;*----------------------------------------------------------------------
	;* Status PROGRAMMING/Stato PROGRAMMAZIONE
	;*----------------------------------------------------------------------
	cpi	z_status,s_programming_exit
				;    elsif status = programming/
				;    altrimenti se stato = programmazione
	brlo	l_end_check_prog;    then/allora
	cpi	z_status,s_programming_auto_guard
				;      if status = programming_auto_guard/
				;      se stato = programmazione auto guardia
	brne	l_enter_inactive;      then/allora
	clr	z_tmp1		;        reset auto guard timer/
				;        azzera temporizzazione auto guardia
	ldi	z_tmp,e_turn_on_timer_1
	rcall	l_write_eeprom
	ldi	z_tmp,e_turn_on_timer_0
	rcall	l_write_eeprom
l_enter_inactive:		;      fi/fine se
	rcall	l_inactive_diagn;      enter status inactive/
				;      entra in stato inattivo
	rjmp	l_end_alarm_tim	;
l_end_check_prog:		;
				;  fi/fine se
l_end_alarm_tim:		;fi/fine se
.endmacro


.macro m_alarm_eh
;*==============================================================================
;* Alarm event handler/Gestore eventi allarme
;*==============================================================================
	;*----------------------------------------------------------------------
	;* Needs:/Ha bisogno di:
	;*----------------------------------------------------------------------
;To be executed after every clock interrupt/
;Essere eseguito dopo ogni interruzione di clock
;*==============================================================================
;* Check input status changes and validate if persistency check passed/
;* Controlla i cambi di stato in ingresso e convalidali se il controllo di
;*   persistenza  superato
;*==============================================================================
;l_change_detection_port_b:
	in	z_tmp2,PINB	;read input pins/leggi gli ingressi
	cp	z_tmp2,z_pinsb_val
				;if actual value pins B <> previous value pins B
				;se il valore attuale degli ingressi B  diverso
				;  dal valore precedente
	breq	l_end_chg_detect;then/allora
	cp	z_tmp2,z_pinsb_new
				;  if actual value pins B <> new value pins B/
				;  se il valore attuale degli ingressi B 
				;    diverso dal valore nuovo
	breq	l_decr_persist	;  then/allora
	clr	z_persistency	;    pulse persistency = 256 units = 146 ms/
				;    persistenza impulso = 256 unit = 146 ms
	mov	z_pinsb_new,z_tmp2
				;    new value pins B = actual value pins B/
				;    valore nuovo ingressi B = valore attuale
	rjmp	l_end_chg_detect;  else/altrimenti
l_decr_persist:
	dec	z_persistency	;    persistency = persistency - 1/
				;    persistenza = persistenza - 1
				;    if persistency = 0/se persistenza = 0
	brne	l_end_valid_chg_;    then/allora
	eor	z_tmp2,z_pinsb_val
				;      find sensors which changed state/
				;      trova i sensori che hanno cambiato stato
	mov	z_pinsb_val,z_pinsb_new
				;      store validated value of pins B/
				;      immagazzina il valore convalidato
				;        degli ingressi B
	sbrc	z_pinsd_val,b_power_low
				;      if power is not low/
				;      se l'alimentazione non  bassa
	rjmp	l_end_ck_power	;      then/allora
	mov	z_pinsb_alarm,z_tmp2
				;        store bitmap of sensors in alarm/
				;        immagazzina la mappa di bit dei sensori
				;          in allarme

	;*----------------------------------------------------------------------
	;* Status INACTIVE/Stato INATTIVO
	;*----------------------------------------------------------------------
	cpi	z_status,s_inactive	;if status = inactive/
					;se stato = inattivo
	brne	l_end_chk_inact		;then/allora
	rcall	l_calc_alm_timer_ret	;  reset status timer/
					;  ricarica temporizzatore di stato
	rjmp	l_end_pinb		;
l_end_chk_inact:			;

	;*----------------------------------------------------------------------
	;* Status EXIT/Stato USCITA
	;*----------------------------------------------------------------------
	cpi	z_status,s_exit		;elsif status = exit/
					;altrimenti se stato = uscita
	brne	l_end_chk_exit		;then/allora
	rcall	l_exit			;  enter status exit
					;  (issuing two short buzzes and
					;   resetting status timer)/
					;  entra in stato uscita
					;  (emettendo due suoni corti e
					;   ricaricando il temporizzatore di
					;   stato)
	rjmp	l_end_pinb		;
l_end_chk_exit:				;

	;*----------------------------------------------------------------------
	;* Status GUARD/Stato GUARDIA
	;*----------------------------------------------------------------------
	cpi	z_status,s_guard	;elsif status = guard/
					;altrimenti se stato = guardia
	brne	l_end_chk_guard		;then/allora
	rcall	l_pre_alarm		;  enter pre-alarm status/
					;  entra in stato preallarme
	rjmp	l_end_pinb		;
l_end_chk_guard:			;

l_end_pinb:			;        fi/fine se
l_end_ck_power:			;      fi/fine se
l_end_valid_chg_:		;    fi/fine se
l_end_ck_persist:		;  fi/fine se
l_end_chg_detect:		;fi/fine se

.endmacro


;*==============================================================================
;* Enter alarm machine states procedures/
;* Procedure di entrata negli stati della macchina di allarme
;*==============================================================================

	;*----------------------------------------------------------------------
	;* Enter status INACTIVE/Entra in stato INATTIVO
	;*----------------------------------------------------------------------
l_inactive_diagn:
	m_siren_off		;turn off siren in case the status was alarm
				;  (from other states it has no effect)/
				;spegni la sirena nel caso lo stato fosse
				;  allarme (da altri stati non ha effetto)
	m_buzz	0, 0, 1, 0	;start 1 short buzz/inizia 1 bip corto
		;*//////////////////////////////////////////////////////////////
		;* Check diagnostic bytes/controlla i byte diagnostici
		;*//////////////////////////////////////////////////////////////	
;l_check_diagnostic_bytes:
	ldi	z_tmp,e_power_status
l_loop_1_1:			;for last byte to first byte/
				;dall'ultimo byte al primo byte
	rcall	l_read_eeprom	;  read eeprom byte/leggi il byte in eeprom
	tst	z_tmp1		;  if byte <> 0/se il byte  diverso da 0
	brne	l_output_diagn	;  then/allora
				;    enter status inactive output diagnostics/
				;    entra in stato inattivo di emissione
				;      diagnostica
				;  fi/fine se
	dec	z_tmp		;
	cpi	z_tmp,e_sensors_in_alarm
	brpl	l_loop_1_1	;end for/fine da
l_diagnostic_end:
		;*//////////////////////////////////////////////////////////////
		;* End check diagnostic bytes/fine controllo byte diagnostici
		;*//////////////////////////////////////////////////////////////

l_inactive_reset:
	ldi	z_status,s_inactive
				;status = inactive/stato = inattivo
l_calc_alm_timer_ret:
	mov	z_tmp,z_status	;  calculate address of timer in eeprom/
				;  calcola indirizzo temporizzatore in eeprom
	lsl	z_tmp		;  z_tmp = z_tmp*2
	subi	z_tmp,(s_inactive*2)-e_turn_on_timer_1
l_read_alm_timer_ret:
	rcall	l_read_eeprom_word
				;load time from eeprom/carica tempo da eeprom
	mov	z_alarm_timer_0,z_tmp1
	mov	z_alarm_timer_1,z_tmp2
l_ret:
	ret

l_output_diagn:
	ldi	zs_out_address,e_sensors_in_alarm - 1
	ldi	z_status,s_inactive_output_diagn
				;status = inactive output diagn/
				;stato = inattivo di emissione diagnostica
	ldi	zs_tested_bit,0x01
	rjmp	l_output_mem_first_pause
				;start with a long pause/
				;inizia con una pausa lunga
l_output_mem_next_bit:
	lsr	zs_tested_bit	;next bit position/prossima posizione di bit
				;if last bit in byte/se ultimo bit nel byte
	brcc	l_output_mem_same_byte
				;then/allora
	ror	zs_tested_bit	;  zs_tested_bit = 0b10000000
	inc	zs_out_address	;  address := address + 1/
				;  indirizzo = indirizzo + 1
l_output_mem_same_byte:		;fi/fine se
	cpi	zs_out_address,e_power_status + 1
				;if address to be read > last address
				;se indirizzo da leggere > ultimo indirizzo
				;then/allora
	brsh	l_inactive_reset;  enter inactive status/
				;  entra in stato inattivo
				;fi/fine se
	mov	z_tmp,zs_out_address
				;read eeprom byte/leggi byte da eeprom
	rcall	l_read_eeprom	;
	m_buzz	0, 0, 1, 0	;buzzer on for 0.1 s/bip di 0,1 s
	ldi	z_alarm_timer_0,1
				;status timer = 1 s/
				;temporizzatore di stato = 1 s
	and	z_tmp1,zs_tested_bit
				;if tested bit = 1/se bit = 1
	breq	l_output_mem_short
				;then/allora
;l_output_mem_long:
	m_buzz	1, 0, 1, 0	;  buzzer on for 1 s/bip di 1 s
l_output_mem_first_pause:
	ldi	z_alarm_timer_0,2
				;  status timer = 2 s/
				;temporizzatore di stato = 2 s
				;fi/fine se
l_output_mem_short:
l_output_mem_last_bit:
	sbrc	zs_tested_bit,0	;if last bit in byte/
				;se  l'ultimo bit nel byte
				;then/allora
	subi	z_alarm_timer_0,-2
				;  status timer := status timer + 2 s/
				;  temporizzatore di stato =
				;    temporizzatore di stato + 2 s
				;fi/fine se
	clr	z_alarm_timer_1	;clear timer msb/
				;azzera il byte pi significativo del
				;  temporizzatore
	rjmp	l_ret

	;*----------------------------------------------------------------------
	;* Enter status EXIT/entra in stato USCITA
	;*----------------------------------------------------------------------
l_exit:
	ldi	z_status,s_exit ;status = exit/stato = uscita
	m_buzz	0, 0, 2, 0	;generate 2 short buzzes with short pause/
				;genera 2 bip corti con pausa corta
	rjmp	l_calc_alm_timer_ret
				;load status timer/
				;carica temporizzatore di stato

	;*----------------------------------------------------------------------
	;* Enter status GUARD/entra in stato GUARDIA
	;*----------------------------------------------------------------------
l_guard:
	m_siren_off		;turn off siren in case the status was alarm
				;  (from other states it has no effect)/
				;spegni la sirena nel caso lo stato fosse
				;  allarme (da altri stati non ha effetto)
	ldi	z_status,s_guard
				;status = guard/stato = guardia
	m_buzz	0, 0, 3, 0	;start 3 short buzzes - short pauses/
				;inizia 3 bip corti - pause corte
	rjmp	l_ret

	;*----------------------------------------------------------------------
	;* Enter status PRE-ALARM/entra in stato PREALLARME
	;*----------------------------------------------------------------------
l_pre_alarm:
	ldi	z_status,s_pre_alarm
				;status = pre-alarm/stato = preallarme
	m_buzz	0, 0, 4, 0	;start 4 short buzzes - short pauses/
				;inizia 4 bip corti - pause corte
	rjmp	l_calc_alm_timer_ret
				;load status timer/
				;carica temporizzatore di stato

	;*----------------------------------------------------------------------
	;* Enter status ALARM/entra in stato ALLARME
	;*----------------------------------------------------------------------
l_alarm:
	ldi	z_status,s_alarm;status = alarm/stato = allarme
	m_stop_bat_test		;stop battery test if running/
				;ferma prova batteria se  in corso
	m_buzz	0, 0, 5, 0	;start 5 short buzzes - short pauses/
				;inizia 5 bip corti - pause corte
	ldi	z_tmp,e_sensors_in_alarm
	mov	z_tmp2,z_pinsb_alarm
	rcall	l_set_eeprom_bits
				;update bitmap of sensors in alarm in eeprom/
				;aggiorna bit dei sensori in allarme in eeprom
	m_siren_on		;siren on/accendi sirena
	rjmp	l_calc_alm_timer_ret
				;load status timer/
				;carica temporizzatore di stato

	;*----------------------------------------------------------------------
	;* Enter status PROGRAMMING/entra in stato PROGRAMMAZIONE
	;*----------------------------------------------------------------------
l_programming:
	ldi	z_status,s_programming_exit
				;status = programming_exit/
				;stato = programmazione uscita
l_programming_reset_timer:
	ldi	z_alarm_timer_0,255
				;status timer = 255 seconds = 4 m 15 s/
				;temporizzatore di stato = 255 sec. = 4 m 15 s
	ldi	z_alarm_timer_1,0
	rjmp	l_ret
