;****************** HOUSE BURGLAR ALARM / ANTIFURTO PER CASA *******************
;*
;* Title/titolo:	House burglar alarm/Antifurto per casa
;* Version/Versione:	2.0 (version for Elettronica In/
;*			     versione per Elettronica In)
;* Author/Autore:	Guido Bertolotti - guidobertolotti@yahoo.com
;* Target/Destinazione:	AT90S1200
;*
;*                      *** Main file to compile is antif.asm ***
;*                 *** Il file principale da compilare  antif.asm ***
;*
;* Source files are:/ I file sorgenti sono:
;* antif.asm, 1200def.inc, alarm.inc, butt_eh.inc, buzzer.inc, eeprom.inc,
;* int.inc, mai_tim.inc, maint_eh.inc, mainten.inc
;*******************************************************************************
;
;TECHNICAL CHARACTERISTICS OF THE BURGLAR ALARM WITH MICROCONTROLLER
;         Guido Bertolotti - guidobertolotti@yahoo.com
;
;Device type:
;    Home burglar alarm with 8 inputs, wired, with microcontroller.
;
;Disability support:
;    only acoustic signals, which permit the use of the device to the
;      blind people.
;
;Basic functions:
;    programmable exit, entrance and alarm timings;
;    programmable secret codes for activation, deactivation and programming (*);
;    optional quick activation/deactivation, also by button or tele-command
;      contact (not included);
;    signalling of anomalous input status at activation (*);
;    sensors' test on manual activation (walk test);
;    manual exclusion of alarm actuators for test;
;    automatic periodic battery check-up with black-out simulation under load;
;    auto power off for battery discharge;
;    storing of black-out, discharged battery and power off events;
;    automatic periodic connected sensors' efficiency check (*);
;    memory of alarms;
;    signalling of stored anomalies.
;
;Programmable special functions:
;    automatic activation on exit (*);
;    automatic activation on inactivity;
;    anti-aggression (*);
;    day anti-intrusion (*);
;    night anti-intrusion with automatic activation/disactivation and night
;      return without disturbing (*);
;    alarm limit (programmable per input and global) (*);
;    immediate or delayed alarm (programmable per input) (*);
;    linee 24 ore (programmable per input) (*);
;    alarm on rising, falling or both current edges
;      (programmable per input) (*);
;    input disable (*);
;    alarm activation for single or double pulse (programmable per input)
;      with limited timeout for second pulse reception (*);
;    alarm pulse persistency (*);
;    connected siren detection and alarm for siren's wires cutting (*).
;
;Command remote keyboard (*):
;    with 12 keys, internal or external, with acoustic feedback when key
;      pressed (tactile feedback), connectable with two wires, with anti
;      tamper input and alarm on cutting or short-circuiting the wires;
;      it is possible to connect additional keyboards.
;
;Power supply:
;    220 V a.c., with 12 V minimum 2.2 Ah back-up acid battery.
;
;Power consumption in guard status:
;    10 mA at 12 V (sensors and contacts excluded).
;
;Black-out autonomy:
;    around 1 day (considering 8 sensors or circuits connected, with 10 mA
;    consumption each and 2.2 Ah back-up battery).
;
;Sensors' power supply:
;    two-branches fuse-protected, 9-13 V d.c., 160 mA maximum per branch.
;
;Inputs:
;    8, of which one anti-tamper (*), current-threshold.
;
;Devices connectables to the inputs:
;    normally closed switches in series;
;    normally closed deviators in series (*);
;    normally open switches in parallel;
;    sensors with relay exit;
;    sensors with open collector output;
;    sensors with logic level output;
;    sensors with current consumption variation in case of alarm.
;
;Protections against false alarms:
;    current threshold inputs (0,1 mA, individually increaseable per input);
;    limitation in the number of alarms per input and total (disableable) (*);
;    programmable minimum persistency of the alarm pulse (*);
;    possibility of defining double pulse inputs (*);
;    disabling of inputs when insufficient supply voltage;
;    discharge to ground of the power supply disturbances;
;    galvanic protection against contacts' oxidation.
;
;Alarm output:
;    two insulated deviators each of 3 Ampere, with which it is possible to
;      pilot normal and positive fall sirens, flashers, telephone diallers,
;      radio-command transmitters etc.
;
;Alarm devices power supply:
;    9-13 Volt c. c., 3 Ampere, fuse protected.
;
;Anti-tampering protections:
;    wired connections, safer and more reliable compared to radio links;
;      protection of the central unit against wire cutting, short-circuiting and
;      connection of 220 volt on inputs, keyboard or command button, buzzer and
;      extern siren power output.
;
;Programming data protection:
;    all data are stored in non-volatile memory and are kept even when power
;      supply is missing.
;      The secret codes are modifiable through a specific procedure, but they
;      are not readable (*).
;
;Microcontroller type:
;    Atmel AVR AT90S1200.
;
;Quartz clock frequency:
;    3.579.545 Hz.
;
;(*) only available in the complete software version.
;
;******************************************************************************/
;
;CARATTERISTICHE TECNICHE CENTRALINA ANTIFURTO A MICROCONTROLLORE
;         Guido Bertolotti - guidobertolotti@yahoo.com
;
;Tipo dispositivo:
;    centralina antifurto per casa a 8 ingressi via filo, con microcontrollore.
;
;Supporto disabilit:
;    segnalazioni esclusivamente acustiche, che permettono l'uso del dispositivo
;      ai ciechi.
;
;Funzionalit di base:
;    temporizzazioni programmabili di uscita, ingresso e allarme;
;    codici segreti programmabili per attivazione, disattivazione e
;      programmazione (*);
;    inserimento/disinserimento rapido opzionale, anche tramite pulsante o
;      contatto di telecomando (non incluso);
;    segnalazione stato anomalo degli ingressi all'inserimento (*);
;    prova sensori su attivazione manuale (walk test);
;    esclusione manuale degli attuatori di allarme per prova centralina;
;    prova periodica e automatica della batteria tampone con simulazione
;      black-out sotto carico;
;    spegnimento automatico per scarica batteria (auto power off);
;    memorizzazione eventi di black-out, scarica batteria e spegnimento
;      centralina;
;    controllo periodico automatico efficienza sensori collegati (*);
;    memoria allarmi;
;    segnalazione anomalie memorizzate.
;
;Funzionalit speciali programmabili:
;    attivazione automatica per uscita (*);
;    attivazione automatica per assenza movimenti;
;    antiaggressione (*);
;    antintrusione diurna (*);
;    guardia antintrusione notturna con attivazione/disattivazione automatica e
;      rientro notturno senza disturbare (*);
;    limitazione numero di allarmi (programmabile per ingresso e globale) (*);
;    allarme immediato o ritardato (programmabile per ingresso) (*);
;    linee 24 ore (programmabili per ingresso) (*);
;    allarme su fronte di salita, di discesa o entrambi i fronti della corrente
;      (programmabile per ingresso) (*);
;    disabilitazione ingresso (*);
;    attivazione allarme per singolo o doppio impulso (programmabile per
;      ingresso) con tempo limite programmabile per ricezione secondo
;      impulso (*);
;    persistenza impulso di allarme (*);
;    rilevamento sirena collegata e allarme per taglio fili sirena (*).
;
;Tastiera remota di comando (*):
;    a 12 tasti, interna o esterna, con riscontri acustici alla pressione dei
;      tasti (tactile feedback), collegabile alla centralina con due fili, con
;      interruttore antimanomissione (tamper) e allarme per taglio o
;      cortocircuito fili (escludibile); possibilit di collegare tastiere
;      aggiuntive.
;
;Alimentazione:
;    220 V c.a., con batteria al piombo in tampone da 12 V - 2,2 Ah minimo.
;
;Assorbimento in stato di guardia:
;    10 mA a 12 V (solo centralina: sensori e circuiti contatti esclusi).
;
;Autonomia in caso di black-out:
;    circa 1 giorno (considerando 8 sensori o circuiti contatti collegati con
;      assorbimento di 10 mA ciascuno e batteria da 2,2 Ah).
;
;Alimentazione sensori:
;    a due rami protetti da fusibili, 9-13 V c.c., 160 mA massimi per ramo;
;      corrente aumentabile se necessario.
;
;Ingressi:
;    8, di cui uno antisabotaggio (*), a soglia di corrente.
;
;Dispositivi collegabili agli ingressi:
;    interruttori normalmente chiusi in serie;
;    deviatori normalmente chiusi in serie (*);
;    interruttori normalmente aperti in parallelo;
;    sensori con uscita a rel;
;    sensori con uscita a collettore aperto;
;    sensori con uscita a livello logico;
;    sensori con variazione di assorbimento di corrente in caso di allarme.
;
;Protezioni contro falsi allarmi:
;    ingressi a soglia di corrente (0,1 mA, aumentabile individualmente per
;      ingresso);
;    limitazione numero di allarmi per ingresso e totale (escludibile) (*);
;    persistenza minima programmabile dell'impulso di allarme (*);
;    possibilit di definire sensori a doppio impulso (*);
;    disabilitazione ingressi se tensione di alimentazione insufficiente;
;    scarica a terra dei disturbi di alimentazione;
;    protezione galvanica contro l'ossidazione dei contatti.
;
;Uscita di allarme:
;    due scambi isolati da 3 Ampere ciascuno, con cui  possibile attivare sirene
;      normali e a caduta di positivo, lampeggiatori, combinatori telefonici,
;      trasmettitori per radiocomando, ecc.
;
;Alimentazione attuatori di allarme:
;    9-13 Volt c. c., 3 Ampere, protetta con fusibile.
;
;Protezioni antisabotaggio:
;    collegamenti via filo, pi sicuri e affidabili rispetto a collegamenti via
;      radio; protezione centralina contro taglio fili, cortocircuito e
;      applicazione 220 Volt c.a. su ingressi, tastiera o pulsante di comando,
;      ronzatore piezoelettrico e uscita sirena/avvisatore esterno.
;
;Protezione dati di programmazione:
;    tutti i dati sono memorizzati in memoria non volatile e permangono anche in
;      assenza di alimentazione. I codici segreti sono modificabili con opportuna
;      procedura, ma non leggibili (*).
;
;Tipo microcontrollore:
;    Atmel AVR AT90S1200.
;
;Frequenza di clock quarzata:
;    3.579.545 Hz.
;
;(*) disponibile solo nella versione software completa.
;
;*******************************************************************************
;*
;* Port usage/		:B7..B0: in: detectors/sensori
;* Uso porte		:D6:	out: battery test/prova batteria
;*			:D5:	out: buzzer/ronzatore
;*			:D4:	out: siren/sirena
;*			:D3:	out: power/alimentazione
;*			:D2:	 in: command button/pulsante di comando
;*			:D1:	 in: mains absence/assenza rete
;*			:D0:	 in: power supply low/alimentazione bassa
;*
;*******************************************************************************
;*
;* Naming conventions/Convenzioni di denominazione:
;*
;* z_xxx	global variable/variabile globale
;* zs_xxx	static variable local to module(s) (must not be used by
;*		unauthorized modules)/
;*		variabile statica locale al/ai modulo/i (non deve essere usata
;*              da moduli non autorizzati)
;* zq_xxx	variable local to procedure (can be re-used (even with different
;*              name) by other independently called procedures)/
;*		variabile locale a procedura (pu essere riutilizzata (anche con
;*              nome diverso) da altre procedure chiamate indipendentemente)
;* s_xxx	constant synonym (0..255)/sinonimo costante (0..255)
;* b_xxx	constant bit selector (0..7)/selettore di bit costante (0..7)
;* l_xxx	label/etichetta
;* e_xxx	eeprom address/indirizzo eeprom
;* p_xxx	i/o port/porta di ingresso/uscita
;* m_xxx	macro
;*
;*******************************************************************************

.include "1200def.inc"

;*******************************************************************************
;*
;* Constants definitions/Definizione delle costanti
;*
;*******************************************************************************

;*==============================================================================
;* Timer interrupt constants/Costanti dell'interruzione del temporizzatore
;*==============================================================================

.equ	s_f_xtal	=3579545	;quartz oscillator frequency in Hz/
					;frequenza dell'oscillatore a quarzo
					;in Hz
.equ	s_presc		=8		;chosen hardware prescaler value/
					;valore del divisore hardware scelto
.equ	s_presc_bmp	=0b00000010	;corresponding prescaler bitmap/
					;mappa corrispondente del divisore
.equ	s_time_base_1	=100		;time base 1 frequency (Hz)/
					;frequenza della base dei tempi 1 (Hz)
.equ	s_timer_1_presc	=(s_f_xtal+s_presc*128*s_time_base_1)/(s_presc*256*s_time_base_1)
					;reload value for prescaler 1/
					;valore di ricarica del divisore 1
.equ	s_time_base_2	=1		;time base 2 frequency (Hz)
					;frequenza della base dei tempi 2 (Hz)
.equ	s_timer_2_presc	=(s_f_xtal+s_presc*128*s_timer_1_presc*s_time_base_2)/(s_presc*256*s_timer_1_presc*s_time_base_2)
					;reload value for timer 2 prescaler
					;valore di ricarica del divisore 2

;*==============================================================================
;* Port B is used as input port for alarm sensors/
;* La porta B  usata come ingresso per i sensori di allarme
;*==============================================================================
.equ	b_sensor1	=7
.equ	b_sensor2	=6
.equ	b_sensor3	=5
.equ	b_sensor4	=4
.equ	b_sensor5	=3
.equ	b_sensor6	=2
.equ	b_sensor7	=1
.equ	b_sensor8	=0

.equ	s_portb_inputs	=0b11111111
.equ	s_portb_outputs	=0b00000000

;*==============================================================================
;* Port D is used as miscellaneous input/output port/
;* La porta D  usata come ingressi/uscite varie
;*==============================================================================
.equ	p_buzzer	=PORTD
.equ	b_power_low_in	=0
.equ	b_mains_absence	=1
.equ	b_command	=2
.equ	b_power		=3
.equ	b_siren		=4
.equ	b_buzzer	=5
.equ	b_battery_test	=6

.equ	s_portd_inputs	=0b00000111
.equ	s_portd_outputs	=0b11111000

;*==============================================================================
;* EEPROM addresses/Indirizzi EEPROM
;*==============================================================================
.eseg
e_sensors_in_alarm:	.db 0		;bitmap of sensors which went in alarm/
					;mappa dei sensori andati in allarme
e_power_status:		.db 0xD0	;bitmap of power supply alarms:/
					;mappa degli allarmi di alimentazione:

.equ	b_mains_failure	=7			;bit 7 (msb)
					;Meaning:/Significato:
					;Mains failure/Assenza rete
					;Probable causes:/Cause probabili:
					;Mains supply was gone for at least a
					;few seconds./L'alimentazione di rete 
					;mancata per almeno qualche secondo.

.equ	b_power_low	=6			;bit 6
					;Meaning:/Significato:
					;Power supply low voltage/Bassa tensione
					;di alimentazione
					;Probable causes:/Cause probabili:
					;If it is the only anomaly: temporary
					;short circuit, strong power supply
					;disturbances.
					;If associated with mains failure and
					;microcontroller restart, then the
					;battery is discharged (just wait
					;for automatic recharge)./
					;Se  l'unica anomalia: corto circuito
					;temporaneo, forti disturbi di
					;alimentazione.
					;Se associato con mancanza alimentazione
					;di rete e reinizializzazione del
					;microcontrollore, significa che la
					;batteria  scarica (attendere
					;semplicemente la ricarica automatica).

.equ	b_broken_battery=b_power_low - 1	;bit 5
					;Meaning:/Significato:
					;Periodic battery check-up unsuccessful/
					;Prova periodica della batteria fallita
					;Probable causes:/Cause probabili:
					;Battery is exausted (residual capacity
					;less than 600 mAh) and must be changed
					;with a new one;
					;or battery disconnected, interrupted
					;battery fuse, oxidated battery
					;connections./
					;La batteria  esausta (capacit residua
					;di meno di 600 mAh) e deve essere
					;sostituita con una nuova;
					;oppure batteria staccata, fusibile
					;batteria interrotto, morsetti batteria
					;ossidati.

.equ	b_micro_restart	=4			;bit 4
					;Meaning:/Significato:
					;Microcontroller restart/
					;Reinizializzazione microcontrollore
					;Probable causes:/Cause probabili:
					;Power off, auto power off,
					;strong disturbances, software error./
					;Spegnimento, autospegnimento, forti
					;disturbi, errore di programmazione.

						;bits 3..0 fixed to 0

e_turn_on_timer_1:	.db 0xFD	;automatic turn-on timer	(msb)/
					;tempo di accensione automatica	(byte
					;pi significativo)
e_turn_on_timer_0:	.db 0x20	;automatic turn-on timer	(lsb)/
					;tempo di accensione automatica	(byte
					;meno significativo)
					;(64800 s = 18 h)
e_exit_timer_1:		.db 0		;exit timer			(msb)/
					;tempo di uscita		(byte
					;pi significativo)
e_exit_timer_0:		.db 60		;exit timer			(lsb)/
					;tempo di uscita		(byte
					;meno significativo)
					;(60 s = 1 m)
e_guard_timer_1:	.db 0x00	;guard timer (not used, empty)	(msb)/
					;tempo di guardia (non usato, vuoto)
					;(byte pi significativo)
e_guard_timer_0:	.db 0x00	;guard timer (not used, empty)	(lsb)/
					;tempo di guardia (non usato, vuoto)
					;(byte meno significativo)				
					;
e_pre_alarm_timer_1:	.db 0		;pre-alarm timer		(msb)/
					;tempo di preallarme		(byte
					;pi significativo)
e_pre_alarm_timer_0:	.db 20		;pre-alarm timer		(lsb)/
					;tempo di preallarme		(byte
					;meno significativo)
					;(20 s)
e_alarm_timer_1:	.db 0		;alarm timer			(msb)/
					;tempo di allarme		(byte
					;pi significativo)
e_alarm_timer_0:	.db 180		;alarm timer			(lsb)/
					;tempo di allarme		(byte
					;meno significativo)
					;(180 s = 3 m)

;*******************************************************************************
;*
;* Variables definitions/Definizioni delle variabili
;*
;*******************************************************************************

;*==============================================================================
;* Register global variables/Variabili globali in registri
;*==============================================================================
	;*----------------------------------------------------------------------
	;* Low registers (0..15)/Registri bassi (0..15)
	;*----------------------------------------------------------------------
.def	z_pinsd_val	=R1	;validated value of PORTD pins/
				;valore convalidato della porta D
.def	z_pinsd_new	=R2	;new value of PORTD pins (to be validated)/
				;nuovo valore della porta D (da convalidare)
.def	z_pinsb_val	=R3	;validated (persistent) value of PORTB pins/
				;valore convalidato (persistente) della porta B
.def	z_pinsb_new	=R4	;new value of PORTB pins (to be validated)/
				;nuovo valore della porta B (da convalidare)
.def	z_persistency	=R5	;persistency for PORTB input pins validation/
				;persistenza per la convalida dello stato della
				;porta B
.def	z_timer_prescaler_2=R6	;prescaler used for slower (1 s based) timers/
				;predivisore usato per temporizzatori pi lenti
				;(basati su 1 s)
.def	z_tmp1		=R7	;temporary storage for general use/
				;immagazzinamento temporaneo di uso generale
.def	z_bat_test_status=R8	;status of maintenance state machine/
				;stato della macchina di manutenzione
.def	z_int_count	=R9	;interrupt counter/contatore di interruzioni

	;*----------------------------------------------------------------------
	;* High registers (16..31)/Registri alti (16..31)
	;*----------------------------------------------------------------------
.def	zs_flags	=R16	;various flags/flag vari
.equ	b_pressed	=0	;pressed button flag/flag di pulsante premuto
.def	zs_out_address	=R17	;eeprom output address/indirizzo di uscita
				;eeprom
.def	zs_last_address	=R18	;eeprom output last address/ultimo indirizzo di
				;uscita eeprom
.def	z_tmp		=R19	;temporary storage for general use/
				;immagazzinamento temporaneo di uso generale
.def	z_tmp2		=R20	;temporary storage for general use/
				;immagazzinamento temporaneo di uso generale
.def	z_timer_prescaler_1=R21	;prescaler used for faster (10 ms based) timers/
				;predivisore usato per temporizzatori pi veloci
				;(basati su 10 ms)
.def	z_pinsb_alarm	=R22	;bitmap of pin which last went in alarm
				;(register shared with zs_tested_bit)/
				;mappa degli ultimi ingressi andati in allarme
				;(registro condiviso con zs_tested_bit)
.def	zs_tested_bit	=R22	;tested bit when outputting diagnostics
				;(register shared with z_pinsb_alarm)/
				;bit testato all'emissione diagnostica
				;(registro condiviso con z_pinsb_alarm)
.def	z_status	=R23	;status of alarm state machine/
				;stato della macchina di allarme
.def	z_alarm_timer_0	=R24	;alarm timer, counter byte 0/
				;temporizzatore di allarme, byte contatore 0
.def	z_alarm_timer_1	=R25	;alarm timer, counter byte 1/
				;temporizzatore di allarme, byte contatore 1
.def	z_maint_timer_0	=R26	;maintenance timer, counter byte 0/
				;temporizzatore di manutenzione, byte 0
.def	z_maint_timer_1	=R27	;maintenance timer, counter byte 1/
				;temporizzatore di manutenzione, byte 1
.def	zs_buzzer_timer	=R28	;buzzer timer - buzzer.inc/
				;temporizzatore del ronzatore - buzzer.inc
.def	zs_buzzer	=R29	;buzzer handling variable - buzzer.inc/
				;variabile di gestione del ronzatore- buzzer.inc

;*==============================================================================
;* Register local variables/Variabili locali in registri
;*==============================================================================
	;*----------------------------------------------------------------------
	;* Low registers (0..15)
	;*----------------------------------------------------------------------
.def	zq_sr		=R0	;store status register/registro di
				;memorizzazione dello stato
;*******************************************************************************
;*
;* PROGRAM STARTS HERE/IL PROGRAMMA INIZIA QUI
;*
;*******************************************************************************
.LISTMAC			;enable macro expansion/abilita l'espansione
				;delle macro
.cseg

;*==============================================================================
;* Reset vector/Vettore di reinizializzazione
;*==============================================================================

	.org	$0000
	rjmp	l_main		;initialization and main program/
				;inizializzazione e programma principale

;*==============================================================================
;* Interrupts (must be inserted immediately after the reset vector)/
;* Interruzioni (deve essere inserito subito dopo il vettore di
;* reinizializzazione)
;*==============================================================================
.include "int.inc"

;*==============================================================================
;* Buzzer setting procedures and macros/
;* Procedure e macro di configurazione ronzatore
;*==============================================================================
.include "buzzer.inc"

;*==============================================================================
;* Read/Write EEPROM procedures/Procedure di lettura/scrittura EEPROM
;*==============================================================================
.include "eeprom.inc"

;*==============================================================================
;* Maintenance status machine/Macchina a stati di manutenzione
;*==============================================================================
.include "mainten.inc"

;*==============================================================================
;* Alarm status machine/Macchina a stati di allarme
;*==============================================================================
.include "alarm.inc"		;alarm status machine macros and procedures/
				;macro e procedure della macchina di allarme

;*******************************************************************************
;*
;* Initialization and main program/Inizializzazione e programma principale
;*
;*******************************************************************************

l_main:
;*==============================================================================
;* Initialization/Inizializzazione
;*==============================================================================

	;*----------------------------------------------------------------------
	;* Buzzer initialization/Inizializzazione ronzatore
	;*----------------------------------------------------------------------
	m_buzzer_init

	;*----------------------------------------------------------------------
	;* Maintenance initialization/Inizializzazione manutenzione
	;*----------------------------------------------------------------------
	m_maint_init

	;*----------------------------------------------------------------------
	;* Alarm initialization/Inizializzazione allarme
	;*----------------------------------------------------------------------
	m_alarm_init

	;*----------------------------------------------------------------------
	;* Interrupts initialization/Inizializzazione interruzioni
	;*----------------------------------------------------------------------
	m_int_init

	;*----------------------------------------------------------------------
	;* Watchdog initialization/Inizializzazione cane da guardia
	;*----------------------------------------------------------------------
	ldi	z_tmp,0b00001111
	out	WDTCR,z_tmp	;enable watchdog, timeout=2048 kcycles (2048 ms)
				;abilita il cane da guardia, scadenza = 2048
				;kilocicli (2048 ms)

;*==============================================================================
;* Main loop/Ciclo principale
;*==============================================================================
l_main_loop:
	wdr			;watchdog reset/azzera cane da guardia

;******************** code executed as often as possible: *********************/
;****************** codice eseguito il pi spesso possibile: ******************

;l_event_handlers:
	;*----------------------------------------------------------------------
	;* Maintenance event handler/Gestore degli eventi di manutenzione
	;*----------------------------------------------------------------------
.include "maint_eh.inc"

	;*----------------------------------------------------------------------
	;* Command button event handler/Gestore degli eventi del pulsante
	;*----------------------------------------------------------------------
.include "butt_eh.inc"

	tst	z_int_count
	breq	l_main_loop
	dec	z_int_count

;************************ code executed every 0.572 ms: ***********************/
;*********************** codice eseguito ogni 0.572 ms: ***********************

	;*----------------------------------------------------------------------
	;* Alarm event handler/Gestore degli eventi di allarme
	;*----------------------------------------------------------------------
	m_alarm_eh

;l_end_event_handlers:

;l_timers_handlers:
	dec	z_timer_prescaler_1
	brne	l_main_loop_very_near

;l_check_timer_1:
	ldi	z_timer_prescaler_1,s_timer_1_presc
				;reload to obtain a period of 10 ms/
				;ricarica per ottenere un periodo di 10 ms

;************************ code executed every 10 ms: **************************/
;*********************** codice eseguito ogni 10 ms: **************************

	;*----------------------------------------------------------------------
	;* Check buzzer timer: 10 ms .. 2,55 s/
	;* Controlla il temporizzatore del ronzatore: 10 ms .. 2,55 s
	;*----------------------------------------------------------------------
	m_buzzer_timer		;handle buzzer timer/gestisci il temporizzatore
				;del ronzatore
	dec	z_timer_prescaler_2
l_main_loop_very_near:
	brne	l_main_loop_near
;l_check_timer_2:
	ldi	z_tmp,s_timer_2_presc
	mov	z_timer_prescaler_2,z_tmp
				;reload to obtain a period of 1 s
				;ricarica per ottenere un periodo di 1 s

;************************ code executed every second: *************************/
;*********************** codice eseguito ogni secondo: ************************

	;*----------------------------------------------------------------------
	;* Check alarm timer: 1 s .. 18 h 12 m 15 s/
	;* Controlla il temporizzatore di allarme: 1 s .. 18 h 12 m 15 s
	;*----------------------------------------------------------------------
	m_alarm_timer

	;*----------------------------------------------------------------------
	;* Check maint timer: 1 s .. 18 h 12 m 16 s (used for maintenance timing
	;* and power off timer: 4 m 15 s (persistency of low battery voltage))/
	;* Controlla il temporizzatore di manutenzione:  1 s .. 18 h 12 m 16 s
	;* (usato per la temporizzazione di manutenzione e la temporizzazione di
	;* autospegnimento: 4 m 15 s (persistenza della batteria bassa)).
	;*----------------------------------------------------------------------
.include "mai_tim.inc"

;l_end_timers_handlers:
l_main_loop_near:
	rjmp	l_main_loop	;loop forever/cicla per sempre

;*******************************************************************************
;*
;* END/FINE
;*
;*******************************************************************************
