;****************** HOUSE BURGLAR ALARM / ANTIFURTO PER CASA *******************
;*
;* Title/titolo:	House burglar alarm/Antifurto per casa
;* Version/Versione:	2.0 (version for Elettronica In/
;*			     versione per Elettronica In)
;* Author/Autore:	Guido Bertolotti - guidobertolotti@yahoo.com
;* Target/Destinazione:	AT90S1200
;*
;*                      *** Main file to compile is antif.asm ***
;*                 *** Il file principale da compilare  antif.asm ***
;*
;*******************************************************************************

;*==============================================================================
;* Buzzer/buzzer timer macros and procedures - Interface description
;*==============================================================================
	;*----------------------------------------------------------------------
	;* Needs:
	;*----------------------------------------------------------------------

;zs_buzzer		high register
;zs_buzzer_timer	high register
;p_buzzer		port where the buzzer is connected
;b_buzzer		bit in port_buzzer where the buzzer is connected
;m_buzzer_timer		must be executed every 10 ms

	;*----------------------------------------------------------------------
	;* Gives:
	;*----------------------------------------------------------------------

;*.macro m_buzzer_init
;*******************************************************************************
;* Initializes the buzzer
;* Must be called at program's beginning
;* example:
;* m_buzzer_init
;*******************************************************************************

;*.macro m_buzz
;*******************************************************************************
;* Generates a buzz sequence
;* m_buzz buzz_length, pause_length, repetitions, skip first buzz
;*	buzz_length     : (0=short, 1=long)
;*	pause_length    : (0=short, 1=long)
;* 	repetitions     : (1..15)
;*      skip first buzz : (0=don't skip, 1=skip and start with pause)
;* example:
;* m_buzz 0, 1, 3, 0 = short buzz, long pause, 3 times, start with buzz
;*******************************************************************************

;*==============================================================================
;* Buzzer/buzzer timer macros and procedures - Body description
;*==============================================================================

.equ	b_act_buzzer	=0
.equ	b_pause_length	=1
.equ	b_buzz_length	=2
.equ	b_buzzer_off	=3
.equ	s_buzzer_timer_short	= 10 ( 100 ms)
.equ	s_buzzer_timer_long	=100 (1000 ms)
.equ	s_pause_timer_short	= 10 ( 100 ms)
.equ	s_pause_timer_long	=100 (1000 ms)

	;*----------------------------------------------------------------------
	;* Buzzer macros
	;*----------------------------------------------------------------------
		;*//////////////////////////////////////////////////////////////
		;* Timer handling
		;*//////////////////////////////////////////////////////////////

.macro m_buzzer_timer
;*******************************************************************************
;* Check buzzer timer: 10 ms .. 2,55 s
;* this macro must be executed every 10 ms
;*******************************************************************************
	sbrc	zs_buzzer,b_act_buzzer
				;if b_act_buzzer
				;then
	ldi	zs_buzzer_timer,1
				;  buzzer timer := 1 (force timer expiration)
				;fi
	tst	zs_buzzer_timer	;if buzzer timer active
	breq	l_end_timer	;then
	dec	zs_buzzer_timer	;  decrement buzzer timer
	brne	l_end_timer	;  if timer expired
				;  then
l_buzzer_tim_exp:
	cbr	zs_buzzer,1<<b_act_buzzer
	cbi	p_buzzer,b_buzzer
				;    buzzer off
	sbrc	zs_buzzer,b_buzzer_off
				;    if buzzer_off flag
				;    then
	sbi	p_buzzer,b_buzzer
				;      buzzer on
				;    fi
	subi	zs_buzzer,1<<3	;    if buzzer_off flag
				;    then
				;      buzzer_off := 0
				;    else
				;      buzzer_off := 1
				;      decrement buzz counter
				;    fi
	brcs	l_end_timer	;    if buzz counter >= 0
				;    then
	ldi	zs_buzzer_timer,s_buzzer_timer_short
				;      buzzer timer short
	sbrc	zs_buzzer,b_buzz_length
				;      if buzz must be long
				;      then
	ldi	zs_buzzer_timer,s_buzzer_timer_long
				;        buzzer timer long
				;      fi
	sbrs	zs_buzzer,b_buzzer_off
				;      if buzzer_off flag
	rjmp	l_end_timer	;      then	
	ldi	zs_buzzer_timer,s_pause_timer_short
				;        pause timer short
	sbrc	zs_buzzer,b_pause_length
				;        if pause must be long
				;        then
	ldi	zs_buzzer_timer,s_pause_timer_long
				;          pause timer long
				;        fi
				;      fi
				;    fi
				;  fi
l_end_timer:			;fi
.endmacro

		;*//////////////////////////////////////////////////////////////
		;* Interface macros
		;*//////////////////////////////////////////////////////////////

.macro m_buzzer_init
;*******************************************************************************
;* Initializes the buzzer
;* Must be called at program's beginning
;* example:
;* m_buzzer_init
;*******************************************************************************
	ldi	zs_buzzer,(1<<b_act_buzzer)

.endmacro

.macro m_buzz
;*******************************************************************************
;* Generates a buzz sequence
;* m_buzz buzz_length, pause_length, repetitions, skip first buzz
;*	buzz_length     : (0=short, 1=long)
;*	pause_length    : (0=short, 1=long)
;* 	repetitions     : (1..15)
;*      skip first buzz : (0=don't skip, 1=skip and start with pause)
;* example:
;* m_buzz 0, 1, 3, 0 = short buzz, long pause, 3 times, start with buzz
;*******************************************************************************
	ldi	zs_buzzer,(@0<<b_buzz_length)+(@1<<b_pause_length)+((@2-1)<<4)+((1-@3)<<b_buzzer_off)+(1<<b_act_buzzer)
.endmacro
